@@ -18,6 +18,8 @@ use Data::Dumper;
 use File::Basename qw(basename);
 use List::Util qw(first);
 use List::MoreUtils qw(uniq);
+use Try::Tiny;
+use ExtUtils::PkgConfig;
 
 use XML::Simple qw(:strict);
 
@@ -28,6 +30,7 @@ sub slurp {
 	<$fh>;
 }
 
+my $xcb_util_present = 0;
 
 my $prefix = 'xcb_';
 
@@ -515,6 +518,7 @@ sub do_replies($\%\%)
         print OUT "    HV * inner_hash;\n";
         print OUT "    AV * alist;\n";
         print OUT "    int c;\n";
+        print OUT "    int _len;\n";
         print OUT "    $cookie cookie;\n";
         print OUT "    $reply *reply;\n";
         print OUT "  CODE:\n";
@@ -541,6 +545,14 @@ sub do_replies($\%\%)
             my $iterator_next = mangle($list->{type}) . '_next';
             my $pre = mangle($req->{name});
 
+            if ($list->{type} eq 'void') {
+                # A byte-array. Provide it as SV.
+                print OUT "    _len = reply->value_len * (reply->format / 8);\n";
+                print OUT "    if (_len > 0)\n";
+                print OUT "        hv_store(hash, \"value\", strlen(\"value\"), newSVpvn((const char*)(reply + 1), _len), 0);\n";
+                next;
+            }
+
             # Get the type description of the list’s members
             my $struct = first { $_->{name} eq $list->{type} } @{$xcb->{struct}};
 
@@ -645,6 +657,13 @@ sub generate {
 
     -d $path or die "$path: $!\n";
 
+    # check if xcb-util could be found, if so, we are using the version after the repository split
+    try {
+        my %info = ExtUtils::PkgConfig->find("xcb-util");
+        print "xcb-util present\n";
+        $xcb_util_present = 1;
+    };
+
     # TODO: Handle all .xmls
     #opendir(DIR, '.');
     #@files = grep { /\.xml$/ } readdir(DIR);
@@ -683,10 +702,14 @@ eot
         print OUT qq|#include "$bname.typedefs"\n|;
     }
 
-    print OUT << 'eot';
+if ($xcb_util_present) {
+    print OUT "typedef xcb_icccm_wm_hints_t X11_XCB_ICCCM_WMHints;\n";
+} else {
+    print OUT "typedef xcb_wm_hints_t X11_XCB_ICCCM_WMHints;\n";
+}
 
+    print OUT << 'eot';
 typedef struct my_xcb_conn XCBConnection;
-typedef xcb_wm_hints_t X11_XCB_ICCCM_WMHints;
 typedef xcb_size_hints_t X11_XCB_ICCCM_SizeHints;
 typedef int intArray;
 
@@ -697,9 +720,65 @@ intArray *intArrayPtr(int num) {
 
         return array;
 }
+
+static SV *
+_new_event_object(xcb_generic_event_t *event)
+{
+    int type;
+    char *objname;
+    HV* hash = newHV();
+
+    hv_store(hash, "response_type", strlen("response_type"), newSViv(event->response_type), 0);
+    hv_store(hash, "sequence", strlen("sequence"), newSViv(event->sequence), 0);
+
+    // Strip highest bit (set when the event was generated by another client)
+    type = (event->response_type & 0x7F);
+
+    switch (type) {
+        case XCB_MAP_NOTIFY:
+        {
+            objname = "X11::XCB::Event::MapNotify";
+            xcb_map_notify_event_t *e = (xcb_map_notify_event_t*)event;
+            hv_store(hash, "event", strlen("event"), newSViv(e->event), 0);
+            hv_store(hash, "window", strlen("window"), newSViv(e->window), 0);
+            hv_store(hash, "override_redirect", strlen("override_redirect"), newSViv(e->override_redirect), 0);
+        }
+        break;
+
+        case XCB_FOCUS_IN:
+        case XCB_FOCUS_OUT:
+        {
+            objname = "X11::XCB::Event::Focus";
+            xcb_focus_in_event_t *e = (xcb_focus_in_event_t*)event;
+            hv_store(hash, "event", strlen("event"), newSViv(e->event), 0);
+            hv_store(hash, "mode", strlen("mode"), newSViv(e->mode), 0);
+        }
+        break;
+
+        case XCB_CLIENT_MESSAGE:
+        {
+            objname = "X11::XCB::Event::ClientMessage";
+            xcb_client_message_event_t *e = (xcb_client_message_event_t*)event;
+            hv_store(hash, "window", strlen("window"), newSViv(e->window), 0);
+            hv_store(hash, "type", strlen("type"), newSViv(e->type), 0);
+            hv_store(hash, "data", strlen("data"), newSVpvn(&(e->data), 20), 0);
+        }
+        break;
+
+        default:
+            objname = "X11::XCB::Event::Generic";
+            break;
+    }
+
+
+    return sv_bless(newRV_noinc((SV*)hash), gv_stashpv(objname, 1));
+}
 eot
 
     my $ext = slurp('XCB_util.inc');
+    if (!$xcb_util_present) {
+        $ext =~ s/xcb_icccm_/xcb_/g;
+    }
     print OUT $ext;
 
     print OUT << 'eot';
@@ -751,7 +830,36 @@ has_error(self)
   OUTPUT:
     RETVAL
 
+int
+get_file_descriptor(self)
+    XCBConnection * self
+  CODE:
+    RETVAL = xcb_get_file_descriptor(self->conn);
+  OUTPUT:
+    RETVAL
+
 eot
+    for my $function (qw(wait_for_event poll_for_event)) {
+        print OUT << "eot";
+SV *
+$function(self)
+    XCBConnection * self
+  PREINIT:
+    HV * hash;
+    SV * result;
+    xcb_generic_event_t * event;
+  CODE:
+    event = xcb_$function(self->conn);
+    if (event == NULL) {
+        RETVAL = &PL_sv_undef;
+    } else {
+        RETVAL = _new_event_object(event);
+    }
+  OUTPUT:
+    RETVAL
+
+eot
+    }
 
     foreach my $name (@files) {
         my $path = $name;
